
#include "cperfdmf.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>



int main(int argc, char** argv) {
  int i;
  int selection;
  int ce;

  if (argc < 4) {
    fprintf (stderr, "Usage: %s <directory> <experiment name> <trial name>\n",argv[0]);
    return -1;
  }

  if (CPERFDMF_OK != (ce = cperfdmf_initialize())) {
    fprintf (stderr, "initialization failed: %s\n",cperfdmf_errToString(ce));
    if (CPERFDMF_ERR_INTERNAL_ERROR == ce) {
      fprintf (stderr, "extended error: %s\n",cperfdmf_getExtendedError());
    }
    return -1;
  }

  char perfdmfcfg[2048];
  sprintf (perfdmfcfg,"%s/.ParaProf/perfdmf.cfg",getenv("HOME"));

  if (CPERFDMF_OK != (ce = cperfdmf_connectDB(perfdmfcfg, NULL))) {
    fprintf (stderr, "db initialization failed: %s\n",cperfdmf_errToString(ce));
    if (CPERFDMF_ERR_INTERNAL_ERROR == ce) {
      fprintf (stderr, "extended error: %s\n",cperfdmf_getExtendedError());
    }
    return -1;
  }


  int appToUse = -1;

  cperfdmf_application_t *apps = NULL;
  int numApps;
  if (CPERFDMF_OK != (ce = cperfdmf_getApplications(&apps, &numApps))) {
    fprintf (stderr, "getApplications failed: %s\n",cperfdmf_errToString(ce));
    if (CPERFDMF_ERR_INTERNAL_ERROR == ce) {
      fprintf (stderr, "extended error: %s\n",cperfdmf_getExtendedError());
    }
    return -1;
  }

  for (i=0; i<numApps; i++) {
    if (!strcmp(apps[i].name,"Big Upload")) {
      appToUse = apps[i].appId;
    }
  }

  if (appToUse == -1) {
    // make a new one
    cperfdmf_application_t newApp;
    newApp.name = "Big Upload";
    newApp.appId = -1; // -1 indicates this is new, it will be overwritten when we call save!
    newApp.numFields = 0;
/*     newApp.language = "English"; */
/*     newApp.paraDiag = "I have no idea"; */
/*     newApp.usage = NULL; */
/*     newApp.execOptions = "-a -b -c -d -e -f"; */
/*     newApp.description = "A great application!"; */
/*     newApp.version = "1.1.1"; */

    if (CPERFDMF_OK != (ce = cperfdmf_saveApplication(&newApp))) {
      fprintf (stderr, "saveApplication failed: %s\n",cperfdmf_errToString(ce));
      if (CPERFDMF_ERR_INTERNAL_ERROR == ce) {
	fprintf (stderr, "extended error: %s\n",cperfdmf_getExtendedError());
      }
      return -1;
    }

    appToUse = newApp.appId;
  }


  cperfdmf_experiment_t *exps = NULL;
  int numExps;
  int expToUse = -1;

  if (CPERFDMF_OK != (ce = cperfdmf_getExperiments(&exps, &numExps, appToUse))) {
    fprintf (stderr, "getExperiments failed: %s\n",cperfdmf_errToString(ce));
    if (CPERFDMF_ERR_INTERNAL_ERROR == ce) {
      fprintf (stderr, "extended error: %s\n",cperfdmf_getExtendedError());
    }
    return -1;
  }

  for (i=0; i<numExps; i++) {
    if (!strcmp(exps[i].name,argv[3])) {
      expToUse = exps[i].expId;
    }
  }

  if (expToUse == -1) {
    // make a new one
    cperfdmf_experiment_t newExp;
    
    newExp.appId = appToUse;

/*     newExp.compilerCcName = "compilerCcName"; */
/*     newExp.compilerCcVersion = "compilerCcVersion"; */
/*     newExp.compilerCppName = "compilerCppName"; */
/*     newExp.compilerCppVersion = "compilerCppVersion"; */
    
/*     newExp.compilerJavaDirpath = "compilerJavaDirpath"; */
/*     newExp.compilerJavaVersion = "compilerJavaVersion"; */
/*     newExp.compilerUserData = "compilerUserData"; */
    
/*     newExp.configArchitecture = "configArchitecture"; */
/*     newExp.configCc = "configCc"; */
/*     newExp.configCpp = "configCpp"; */
/*     newExp.configJdk = "configJkd"; */
/*     newExp.configPrefix = "configPrefix"; */
/*     newExp.configProfile = "cfp"; */
/*     newExp.configUserData = "configUserdata"; */
    
    newExp.expId = -1; // -1 indicates this is new, it will be overwritten when we call save!
    
    newExp.name = argv[3];
/*     newExp.systemArch = "systemArch"; */
/*     newExp.systemL1CacheSize = "systemL1CacheSize"; */
/*     newExp.systemL2CacheSize = "systemL2CacheSize"; */
/*     newExp.systemMachineType = "systemMachineType"; */
/*     newExp.systemMemorySize = "systemMemorySize"; */
/*     newExp.systemName = "systemName"; */
/*     newExp.systemOS = "systemOS"; */
/*     newExp.systemProcessorAmount = "systemProcessorAmount"; */
/*     newExp.systemUserData = "systemUserData"; */
/*     newExp.userData = "userData"; */
    
    if (CPERFDMF_OK != (ce = cperfdmf_saveExperiment(&newExp))) {
      fprintf (stderr, "saveExperiment failed: %s\n",cperfdmf_errToString(ce));
      if (CPERFDMF_ERR_INTERNAL_ERROR == ce) {
	fprintf (stderr, "extended error: %s\n",cperfdmf_getExtendedError());
      }
      return -1;
    }
    expToUse = newExp.expId;
  }


  printf ("loading trial!\n");
  
  int trialId;
  if (CPERFDMF_OK != (ce = cperfdmf_uploadTauOutput(&trialId, argv[1], expToUse))) {
    fprintf (stderr, "upload failed: %s\n",cperfdmf_errToString(ce));
    if (CPERFDMF_ERR_INTERNAL_ERROR == ce) {
      fprintf (stderr, "extended error: %s\n",cperfdmf_getExtendedError());
    }
    return -1;
  }
  

  cperfdmf_trial_t newTrial;

  newTrial.appId = appToUse;
  newTrial.expId = expToUse;
  newTrial.trialId = trialId;
  newTrial.name = argv[2];
/*   newTrial.numNodes = 8; */
/*   newTrial.numContextsPerNode = 64; */
/*   newTrial.numThreadsPerContext = 128; */
/*   newTrial.numMetrics = 15; */
/*   newTrial.problemDefinition = "problemDefinition"; */
/*   newTrial.time = "2004-04-10"; */
/*   //  newTrial.time = NULL; */
/*   newTrial.userData = "userData"; */

  if (CPERFDMF_OK != (ce = cperfdmf_saveTrial(&newTrial))) {
    fprintf (stderr, "saveTrial failed: %s\n",cperfdmf_errToString(ce));
    if (CPERFDMF_ERR_INTERNAL_ERROR == ce) {
      fprintf (stderr, "extended error: %s\n",cperfdmf_getExtendedError());
    }
    return -1;
  }


  printf ("loaded ok\n");
  return 0;
}
